@@ -3,56 +3,54 @@ from __future__ import annotations
 import contextlib
 from functools import reduce
 from operator import and_
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Iterator
-from typing import Mapping
-from typing import Sequence
+from typing import TYPE_CHECKING, Any, Iterator, Mapping, Sequence
 
 import duckdb
-from duckdb import FunctionExpression
-from duckdb import StarExpression
+from duckdb import FunctionExpression, StarExpression
 
-from narwhals._duckdb.utils import col
-from narwhals._duckdb.utils import evaluate_exprs
-from narwhals._duckdb.utils import generate_partition_by_sql
-from narwhals._duckdb.utils import lit
-from narwhals._duckdb.utils import native_to_narwhals_dtype
+from narwhals._duckdb.utils import (
+    DeferredTimeZone,
+    col,
+    evaluate_exprs,
+    generate_partition_by_sql,
+    lit,
+    native_to_narwhals_dtype,
+)
+from narwhals._utils import (
+    Implementation,
+    Version,
+    generate_temporary_column_name,
+    not_implemented,
+    parse_columns_to_drop,
+    parse_version,
+    validate_backend_version,
+)
 from narwhals.dependencies import get_duckdb
-from narwhals.exceptions import ColumnNotFoundError
 from narwhals.exceptions import InvalidOperationError
 from narwhals.typing import CompliantLazyFrame
-from narwhals.utils import Implementation
-from narwhals.utils import Version
-from narwhals.utils import generate_temporary_column_name
-from narwhals.utils import not_implemented
-from narwhals.utils import parse_columns_to_drop
-from narwhals.utils import parse_version
-from narwhals.utils import validate_backend_version
 
 if TYPE_CHECKING:
     from types import ModuleType
 
     import pandas as pd
     import pyarrow as pa
-    from typing_extensions import Self
-    from typing_extensions import TypeIs
+    from duckdb import Expression
+    from duckdb.typing import DuckDBPyType
+    from typing_extensions import Self, TypeIs
 
     from narwhals._compliant.typing import CompliantDataFrameAny
     from narwhals._duckdb.expr import DuckDBExpr
     from narwhals._duckdb.group_by import DuckDBGroupBy
     from narwhals._duckdb.namespace import DuckDBNamespace
     from narwhals._duckdb.series import DuckDBInterchangeSeries
+    from narwhals._utils import _FullContext
     from narwhals.dataframe import LazyFrame
     from narwhals.dtypes import DType
     from narwhals.stable.v1 import DataFrame as DataFrameV1
-    from narwhals.typing import AsofJoinStrategy
-    from narwhals.typing import JoinStrategy
-    from narwhals.typing import LazyUniqueKeepStrategy
-    from narwhals.utils import _FullContext
+    from narwhals.typing import AsofJoinStrategy, JoinStrategy, LazyUniqueKeepStrategy
 
 with contextlib.suppress(ImportError):  # requires duckdb>=1.3.0
-    from duckdb import SQLExpression  # type: ignore[attr-defined, unused-ignore]
+    from duckdb import SQLExpression
 
 
 class DuckDBLazyFrame(
@@ -74,7 +72,7 @@ class DuckDBLazyFrame(
         self._native_frame: duckdb.DuckDBPyRelation = df
         self._version = version
         self._backend_version = backend_version
-        self._cached_schema: dict[str, DType] | None = None
+        self._cached_native_schema: dict[str, DuckDBPyType] | None = None
         self._cached_columns: list[str] | None = None
         validate_backend_version(self._implementation, self._backend_version)
 
@@ -125,7 +123,7 @@ class DuckDBLazyFrame(
 
         return DuckDBInterchangeSeries(self.native.select(name), version=self._version)
 
-    def _iter_columns(self) -> Iterator[duckdb.Expression]:
+    def _iter_columns(self) -> Iterator[Expression]:
         for name in self.columns:
             yield col(name)
 
@@ -179,15 +177,12 @@ class DuckDBLazyFrame(
         selection = [val.alias(name) for name, val in evaluate_exprs(self, *exprs)]
         return self._with_native(self.native.aggregate(selection))  # type: ignore[arg-type]
 
-    def select(
-        self,
-        *exprs: DuckDBExpr,
-    ) -> Self:
+    def select(self, *exprs: DuckDBExpr) -> Self:
         selection = (val.alias(name) for name, val in evaluate_exprs(self, *exprs))
         return self._with_native(self.native.select(*selection))
 
     def drop(self, columns: Sequence[str], *, strict: bool) -> Self:
-        columns_to_drop = parse_columns_to_drop(self, columns=columns, strict=strict)
+        columns_to_drop = parse_columns_to_drop(self, columns, strict=strict)
         selection = (name for name in self.columns if name not in columns_to_drop)
         return self._with_native(self.native.select(*selection))
 
@@ -219,23 +214,25 @@ class DuckDBLazyFrame(
 
     @property
     def schema(self) -> dict[str, DType]:
-        if self._cached_schema is None:
-            # Note: prefer `self._cached_schema` over `functools.cached_property`
+        if self._cached_native_schema is None:
+            # Note: prefer `self._cached_native_schema` over `functools.cached_property`
             # due to Python3.13 failures.
-            self._cached_schema = {
-                column_name: native_to_narwhals_dtype(duckdb_dtype, self._version)
-                for column_name, duckdb_dtype in zip(
-                    self.native.columns, self.native.types
-                )
-            }
-        return self._cached_schema
+            self._cached_native_schema = dict(zip(self.columns, self.native.types))
+
+        deferred_time_zone = DeferredTimeZone(self.native)
+        return {
+            column_name: native_to_narwhals_dtype(
+                duckdb_dtype, self._version, deferred_time_zone
+            )
+            for column_name, duckdb_dtype in zip(self.native.columns, self.native.types)
+        }
 
     @property
     def columns(self) -> list[str]:
         if self._cached_columns is None:
             self._cached_columns = (
                 list(self.schema)
-                if self._cached_schema is not None
+                if self._cached_native_schema is not None
                 else self.native.columns
             )
         return self._cached_columns
@@ -303,7 +300,7 @@ class DuckDBLazyFrame(
                 col(f'lhs."{left}"') == col(f'rhs."{right}"')
                 for left, right in zip(left_on, right_on)
             )
-            condition: duckdb.Expression = reduce(and_, it)
+            condition: Expression = reduce(and_, it)
             rel = self.native.set_alias("lhs").join(
                 other.native.set_alias("rhs"),
                 # NOTE: Fixed in `--pre` https://github.com/duckdb/duckdb/pull/16933
@@ -342,7 +339,7 @@ class DuckDBLazyFrame(
     ) -> Self:
         lhs = self.native
         rhs = other.native
-        conditions: list[duckdb.Expression] = []
+        conditions: list[Expression] = []
         if by_left is not None and by_right is not None:
             conditions.extend(
                 col(f'lhs."{left}"') == col(f'rhs."{right}"')
@@ -357,7 +354,7 @@ class DuckDBLazyFrame(
         else:
             msg = "Only 'backward' and 'forward' strategies are currently supported for DuckDB"
             raise NotImplementedError(msg)
-        condition: duckdb.Expression = reduce(and_, conditions)
+        condition: Expression = reduce(and_, conditions)
         select = ["lhs.*"]
         for name in rhs.columns:
             if name in lhs.columns and (
@@ -390,9 +387,8 @@ class DuckDBLazyFrame(
                 )
                 raise NotImplementedError(msg)
             # Sanitise input
-            if any(x not in self.columns for x in subset_):
-                msg = f"Columns {set(subset_).difference(self.columns)} not found in {self.columns}."
-                raise ColumnNotFoundError(msg)
+            if error := self._check_columns_exist(subset_):
+                raise error
             idx_name = generate_temporary_column_name(8, self.columns)
             count_name = generate_temporary_column_name(8, [*self.columns, idx_name])
             partition_by_sql = generate_partition_by_sql(*(subset_))
