@@ -32,6 +32,7 @@ def test_diff(constructor_eager: ConstructorEager) -> None:
 
 
 def test_diff_lazy(constructor: Constructor) -> None:
+    data = {"i": [None, 1, 2, 3, 4], "b": [1, 2, 3, 5, 3], "c": [5, 4, 3, 2, 1]}
     if "pyarrow_table_constructor" in str(constructor) and PYARROW_VERSION < (13,):
         # pc.pairwisediff is available since pyarrow 13.0.0
         pytest.skip()
@@ -41,7 +42,7 @@ def test_diff_lazy(constructor: Constructor) -> None:
         pytest.skip()
     df = nw.from_native(constructor(data))
     result = df.with_columns(c_diff=nw.col("c").diff().over(order_by="i")).filter(
-        nw.col("i") > 0
+        ~nw.col("i").is_null()
     )
     expected = {
         "i": [1, 2, 3, 4],
