@@ -183,14 +183,6 @@ def test_rolling_sum_series(constructor_eager: ConstructorEager) -> None:
                 ValueError, match="window_size must be greater or equal than 1"
             ),
         ),
-        (
-            4.2,
-            None,
-            pytest.raises(
-                TypeError,
-                match="argument 'window_size': 'float' object cannot be interpreted as an integer",
-            ),
-        ),
         (
             2,
             -1,
@@ -198,14 +190,6 @@ def test_rolling_sum_series(constructor_eager: ConstructorEager) -> None:
                 ValueError, match="min_samples must be greater or equal than 1"
             ),
         ),
-        (
-            2,
-            4.2,
-            pytest.raises(
-                TypeError,
-                match="argument 'min_samples': 'float' object cannot be interpreted as an integer",
-            ),
-        ),
         (
             1,
             2,
@@ -214,6 +198,16 @@ def test_rolling_sum_series(constructor_eager: ConstructorEager) -> None:
                 match="`min_samples` must be less or equal than `window_size`",
             ),
         ),
+        (
+            4.2,
+            None,
+            pytest.raises(TypeError, match=r"Expected '.+?', got: '.+?'\s+window_size="),
+        ),
+        (
+            2,
+            4.2,
+            pytest.raises(TypeError, match=r"Expected '.+?', got: '.+?'\s+min_samples="),
+        ),
     ],
 )
 def test_rolling_sum_expr_invalid_params(
@@ -243,14 +237,6 @@ def test_rolling_sum_expr_invalid_params(
                 ValueError, match="window_size must be greater or equal than 1"
             ),
         ),
-        (
-            4.2,
-            None,
-            pytest.raises(
-                TypeError,
-                match="argument 'window_size': 'float' object cannot be interpreted as an integer",
-            ),
-        ),
         (
             2,
             -1,
@@ -258,14 +244,6 @@ def test_rolling_sum_expr_invalid_params(
                 ValueError, match="min_samples must be greater or equal than 1"
             ),
         ),
-        (
-            2,
-            4.2,
-            pytest.raises(
-                TypeError,
-                match="argument 'min_samples': 'float' object cannot be interpreted as an integer",
-            ),
-        ),
         (
             1,
             2,
@@ -274,6 +252,16 @@ def test_rolling_sum_expr_invalid_params(
                 match="`min_samples` must be less or equal than `window_size`",
             ),
         ),
+        (
+            4.2,
+            None,
+            pytest.raises(TypeError, match=r"Expected '.+?', got: '.+?'\s+window_size="),
+        ),
+        (
+            2,
+            4.2,
+            pytest.raises(TypeError, match=r"Expected '.+?', got: '.+?'\s+min_samples="),
+        ),
     ],
 )
 def test_rolling_sum_series_invalid_params(
